
/*
 * $Id: CltOpts.cc,v 1.3 1999/04/25 06:39:14 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "CltOpts.h"
#include "UniqId.h"
#include "Rnd.h"
#include "SharedOpts.h"

#include "globals.h"

CltOpts TheCltOpts;


CltOpts::CltOpts():
	theOrgAddr(this,        "origin <host:port>", "origin host", NetAddr("127.0.0.1", 8080)),
	thePxyAddr(this,        "proxy <host:port>",  "proxy host", NetAddr()),

	thePorts(this,          "ports <port_range>", "ephemeral|range"),
	theConnPoolCap(this,    "conn_pool <int>",    "[initial] connection pool capacity"),
	adjustConnPoolSize(this,"conn_pool_dyn <bool>","dynamically adjust conn pool capacity"),
	theConnEstTimeout(0,    "conn_est_tout <time>","connection establishing timeout"),
	theLaunchWin(this,      "launch_win <time>",  "interval to start all robots"),
	theCoolPhase(this,      "cool_phase <time>",  "minimal cool-off phase duration"),

	theRobotCnt(this,       "robots <int>",       "number of robots"),
	//theReqCcl(0,          "req_ccl <int>",      "concurrent requests per user"),
	theReqRate(this,        "req_rate <rate>",    "user request rate"),

	ignoreFalseHits(this,   "ign_false_hits <bool>","ignore false hits"),
	printFalseMisses(this,  "prn_false_misses <bool>","dump rep headers of false misses"),
	doIOs(this,             "do_ios <bool>",    "do socket ios", true)
{
	theWorld.share(this);
	theConnEstTimeout.argType("time");
}

bool CltOpts::validate() const {
	if (theRobotCnt < 0)
		cerr << ThePrgName << ": unspecified number of robots" << endl;
	else
	if (!TheOpts.theGoal)
		cerr << ThePrgName << ": unspecified goal" << endl;
	else
	if (theWorld.theReqOrder >= 0 && TheOpts.theGoal.theXactComplCnt < 0)
		cerr << ThePrgName << ": must specify total #completions when using req order" << endl;
	else
	if (TheOpts.theXactThinkTime > 0 && theReqRate)
		cerr << ThePrgName << ": xact_think is useless used with req_rate" << endl;
	else
		return theWorld.validate() && OptGrp::validate();
	return false;
}
