
/*
 * $Id: Http.h,v 1.2 1999/04/20 23:10:16 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef X_HTTP_H
#define X_HTTP_H

#include <iostream.h>

#include "Size.h"
#include "UniqId.h"

class HttpHdr {
	public:
		HttpHdr() { reset(); }
		virtual ~HttpHdr() {}

		virtual void reset();

		bool keepAlive() const;

		bool parse(const char *buf, Size sz);

	protected:
		virtual bool parseField();
		bool fieldEq(const char *name, Size len);

	public:
		Size theHdrSize;
		Size theContSize;
		UniqId theXactId;
		double theHttpVer;
		enum { kaNo, kaDefault, kaYes } theKeepAliveConn;

	protected:
		Size theSrchPos;
		const char *theSrchBuf;
		enum { ssFirst, ssSkip, ssFound } theSrchState;
};

class ReqHdr: public HttpHdr {
	public:
		ReqHdr(): theObjId(-1), theETag(-1) {}

		virtual void reset();

		bool foreign() const { return theObjId <= 0 || !theXactId || theETag <= 0; }

	protected:
		virtual bool parseField();

	public:
		int theObjId;
		int theETag;
};

class RepHdr: public HttpHdr {
	public:
		RepHdr(): isCachable(true) {}

		virtual void reset();

	protected:
		virtual bool parseField();

	public:
		bool isCachable;
};

inline ostream &crlf(ostream &os) { return os.write("\r\n", 2); }

#endif
