
/*
 * $Id: Rnd.cc,v 1.2 1999/04/20 23:10:21 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "xstd.h"

#include <iostream.h>

#include "Assert.h"
#include "Rnd.h"

RndGen::State RndGen::TheDefSeed = 1;

#define TheRealSeedCount 60
static const unsigned long TheRealSeed[TheRealSeedCount] = {
	0x991539b0, 0x16a5bce1, 0x6774a4ca, 0x3e01511a, 0x4e508aa5, 0x61048bff,
	0xf5500610, 0x846b710d, 0x6a198923, 0x896a97a5, 0xdb48f92b, 0x14898448,
	0x37ffd0f9, 0xb58bff8e, 0x59e170f5, 0xcf918a39, 0x09378c72, 0x52c7a45f,
	0x8d293e96, 0x1f4fc2ed, 0xc3db71a9, 0x39b44e06, 0xf8a44ee2, 0x4c8b8099,
	0x19edc30f, 0x87bf4bc3, 0xc9b240ca, 0xe9ee4aff, 0x4382aeca, 0x535b6b23,
	0x9a31901a, 0x32d9c004, 0x9b663161, 0x5da1f320, 0xde3b81bd, 0xdf0a6f91,
	0xf103bbdd, 0x48f340d5, 0x7449e544, 0xbeb1db88, 0xab5c58ef, 0x946554d3,
	0x8c2e67e4, 0xeb3d7973, 0xb11ee08a, 0x2d436b58, 0xda672dfb, 0x1588ca58,
	0xe369732c, 0x904f35c5, 0xd7158fa3, 0x6fa6f01d, 0x616e6b61, 0xac94efa6,
	0x36413f5c, 0xc622c260, 0xf5a42a7f, 0x8a88d741, 0xf5ad9cd3, 0x899921cf
};

// a negative seed will be set (to the default value) before first use
// see trial();
RndGen::RndGen(State aSeed = -1) {
	theSeed = aSeed;
}

void RndGen::seed(State aSeed) {
	Assert(aSeed > 0);
	theSeed = aSeed;
	const int h = TheRealSeed[aSeed % TheRealSeedCount];
	theState = h + aSeed/TheRealSeedCount;
	if (theState <= 0)
		theState = h;
}

RndGen::State RndGen::ltrial() {
	// set if still undefined
	if (theSeed <= 0)
		seed(TheDefSeed);

	/*
	 * Compute x[n + 1] = (7^5 * x[n]) mod (2^31 - 1).
	 * From "Random number generators: good ones are hard to find",
	 * Park and Miller, Communications of the ACM, vol. 31, no. 10,
	 * October 1988, p. 1195.
	 */
	const long h = theState;
	const long hi = h / 127773;
	const long lo = h % 127773;
	const long x = 16807 * lo - 2836 * hi;

	return theState = (x <= 0 ? x+0x7fffffff : x);
}


/* RndDistr */

ostream &RndDistr::print(ostream &os) const {
	return os << pdfName() << '(' << mean() << ')';
}
